 /** ---------------------------------------------
  *                 AUDITIVASSIST
  *  ---------------------------------------------
  *  Treball Final de Grau
  *  13 de gener del 2019
  *  Eudald Ferre BarÃƒÂ³
  *  Grau Enginyeria InformÃƒÂ tica
  *  ---------------------------------------------
  *  
  *  Llibreries i codi de tercers utilitzats en aquest projecte:
  *  
  *  - https://github.com/esp8266
  *  - https://github.com/chriscook8/esp-arduino-apboot
  *  - https://github.com/knolleary/pubsubclient
  *  - https://gist.github.com/igrr/7f7e7973366fc01d6393
  *  - https://github.com/armtronix/Wifi-Triac-SSR
  *  - https://github.com/bblanchon/ArduinoJson
  *  
  */

//Active debug, comment to desactive
#define DEBUG
//Select type of node, uncomment to actuator node and comment to sensor node
//#define ACTUATOR

//Define debug functions in debug mode and out of it
#ifdef DEBUG
  #define Debug(x)    Serial.print(x)
  #define Debugln(x)  Serial.println(x)
  #define Debugf(...) Serial.printf(__VA_ARGS__)
  #define Debugflush  Serial.flush
#else
  #define Debug(x)    {}
  #define Debugln(x)  {}
  #define Debugf(...) {}
  #define Debugflush  {}
#endif

//Define input and output pins for actuator and sensor nodes
#ifdef ACTUATOR
  #define OUTPIN 13  //output pin
  #define INPIN -1   //input pin
#else
  #define OUTPIN -1  //output pin
  #define INPIN A0  // Analog input pin
#endif

#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <PubSubClient.h>
#include <ArduinoJson.h>
#include "FS.h"

//***** Settings declare ********************************************************************************************************* 
String hostName ="AUDITIVASSIST"; //The MQTT ID -> MAC adress will be added to make it kind of unique

//##### Object instances ##### 
MDNSResponder mdns;
ESP8266WebServer server(80);
WiFiClient wifiClient;
PubSubClient mqttClient;

//##### Flags ##### They are needed because the loop needs to continue and cant wait for long tasks!
int configToClear=0; // determine if config should be cleared.
boolean inApMode=0;
int _count = 0;

//##### Global vars ##### 
//WiFi Stations HTML list
String st; 
//For MQTT data recieve
char buf[40]; 
//The DNS hostname
char* host;
//Save output state
boolean opState;
//For timers
unsigned long initMs_alarm = 0;
unsigned long prevMs_alarm = 0;
unsigned long initMs_baby = 0;
unsigned long prevMs_baby = 0;
unsigned long initMs_door1 = 0;
unsigned long prevMs_door1 = 0;
unsigned long initMs_door2 = 0;
unsigned long prevMs_door2 = 0;
unsigned long initMs_telephone = 0;
unsigned long prevMs_telephone = 0;
unsigned long prevMs_alerted = 0;
//For alerts
boolean _alarm = false;
boolean _baby = false;
boolean _door1 = false;
boolean _door2 = false;
boolean _telephone = false;

//To be read from Config file
String esid;
String epass;
String alert;

//MQTT parameters
String pubTopic = "AUDITIVASSIST";
String subTopic = "AUDITIVASSIST/#";
String mqttServer = "AUDITIVASSIST-PI";

//-------------- void's -------------------------------------------------------------------------------------
void setup() {
  Serial.begin(115200);
  delay(10);
  
  //Load config
  Debugln("DEBUG: Entering loadConfig()");
  
  if (!SPIFFS.begin()) {
    Serial.println("Failed to mount file system");
  }

  //loadConfig();
  Debugln("DEBUG: loadConfig() passed");

  //Configure inputs and outputs
  pinMode(OUTPIN, OUTPUT);
  opState = false;
  digitalWrite(OUTPIN, opState);

  //Build host name's
  uint8_t mac[6];
  WiFi.macAddress(mac);
  hostName += "-";
  hostName += macToStr(mac);
  hostName.replace(":","-");
  host = (char*) hostName.c_str();

  pubTopic += "/";
  pubTopic += hostName;

  loadConfig();
  Debugln("DEBUG: loadConfig() passed");
  
  // Connect to WiFi network
  Debugln("DEBUG: Entering initWiFi()");
  initWiFi();
  Debugln("DEBUG: initWiFi() passed");
  Debugln("DEBUG: Starting the main loop");
}

//-------------------------------- Main loop ---------------------------
void loop() {

  //If clear config is needed, clear it, and reset node
  if(configToClear==1){
    Debugln("DEBUG: loop() clear config flag set!");
    clearConfig()? Serial.println("Config cleared!") : Serial.println("Config could not be cleared");
    delay(1000);
    ESP.reset();
  }

  if (WiFi.status() == WL_CONNECTED || inApMode){

    server.handleClient();
    delay(1);

    if (!inApMode) {

      if(analogRead(INPIN) > 700){
        Debug("DEBUG: Sound detected!");
        Debug(" level: ");
        Debugln(String(analogRead(INPIN)));
        pubAlert();
      }

      if (!connectMQTT()){
        delay(200);          
      }
          
      mqttClient.loop();
      delay(100);
    
    }
    
  } else {
    Debugln("DEBUG: loop - WiFi not connected");  
    delay(1000);
    initWiFi(); //Try to connect again
  }

  if (_alarm)     { alarmAlert();     }
  if (_baby)      { babyAlert();      }
  if (_door1)     { door1Alert();     }
  if (_door2)     { door2Alert();     }
  if (_telephone) { telephoneAlert(); }

}
