/*
 * Method to open config file in flash memory as json format
 * and read saved parameters
 */
bool loadConfig() {
  //Open config file
  File configFile = SPIFFS.open("/config.json", "r");
  //Throw error and exit if there are any error in open file process
  if (!configFile) {
    Serial.println("Failed to open config file");
    return false;
  }
  //Read file size's
  size_t size = configFile.size();
  //Throw error if file size's is too large
  if (size > 1024) {
    Serial.println("Config file size is too large");
    return false;
  }

  // Allocate a buffer to store contents of the file.
  std::unique_ptr<char[]> buf(new char[size]);

  // We don't use String here because ArduinoJson library requires the input
  // buffer to be mutable
  configFile.readBytes(buf.get(), size);

  //Initialize jsonBuffer and parse config file
  StaticJsonBuffer<400> jsonBuffer;
  JsonObject& json = jsonBuffer.parseObject(buf.get());
  //Throw error is there are any problem 
  if (!json.success()) {
    Serial.println("Failed to parse config file");
    return false;
  }

  //Reads json object values and saves in a local variables
  String esidC = json["esid"];
  String epassC = json["epass"];
  String alertC = json["alert"];

  //Convert local variables in a global variables
  esid = esidC;
  epass = epassC;
  alert = alertC;

  //Prints config readed
  Serial.print("esid: ");
  Serial.println(esid);
  Serial.print("epass: ");
  Serial.println(epass);
  Serial.print("Type of alert: ");
  Serial.println(alert);
  
  return true;
}

/*
 * Method to save paremeters in config file
 */
bool saveConfig() {
  //Initialize jsonBuffer and jsonObject
  StaticJsonBuffer<400> jsonBuffer;
  JsonObject& json = jsonBuffer.createObject();
  //Creates json objects and assign their values
  json["esid"] = esid;
  json["epass"] = epass;
  json["alert"] = alert;

  //Open config file from flash memory
  File configFile = SPIFFS.open("/config.json", "w");
  //Throw error and exit if there are any error in open file process
  if (!configFile) {
    Serial.println("Failed to open config file for writing");
    return false;
  }

  //Save json configuration in config file
  json.printTo(configFile);
  return true;
}

/*
 * Method to clear config saved in config file
 * formating all fash memory
 */
bool clearConfig(){
    Debugln("DEBUG: In config clear!");
    return SPIFFS.format();  
}
