/*
 * Method to connect to a MQTT broker
 */
boolean connectMQTT(){
  //If it's connected return
  if (mqttClient.connected()){
    return true;
  }  
  
  Serial.print("Connecting to MQTT server ");
  Serial.print(mqttServer);
  Serial.print(" as ");
  Serial.println(host);

  //Connects to MQTT broker and subscribes to topics
  if (mqttClient.connect(host)) {
    Serial.println("Connected to MQTT broker");
    if(mqttClient.subscribe((char*)subTopic.c_str())){
      Serial.println("Subsribed to topic.");
    } else {
      Serial.println("NOT subsribed to topic!");      
    }
    return true;
  }
  else {
    Serial.println("MQTT connect failed! ");
    return false;
  }
}

/* Method to disconnect form MQTT broker */
void disconnectMQTT(){
  mqttClient.disconnect();
}

/*
 * Callback function to handle messages arrived
 */
void mqtt_arrived(char* subTopic, byte* payload, unsigned int length) {
  int i = 0;
  
  //Prints message arrived
  Serial.print("MQTT message arrived:  topic: " + String(subTopic));
  
  //Create character buffer with ending null terminator to parse to string
  for(i=0; i<length; i++) {    
    buf[i] = payload[i];
  }
  buf[i] = '\0';
  String msgString = String(buf);
  Serial.println(" message: " + msgString);

  //Check what type of alert has been recived
  //If message recived is "DEVICES" then publish identification of this node
  if (msgString == "DEVICES") {
    pubMessage(hostName);      
  }
  
  if (msgString == "ALARM") {
    _alarm = true;      
  }

  if (msgString == "BABY") {
    _baby = true;      
  }

  if (msgString == "DOOR1") {
    _door1 = true;      
  }

  if (msgString == "DOOR2") {
    _door2 = true;      
  }

  if (msgString == "TELEPHONE") {
    _telephone = true;      
  }
}

/*
 * Method to publish a MQTT message
 */
boolean pubMessage(String message){
  //If MQTT client is not connected, connect   
  if (!connectMQTT()){
      delay(100);
      if (!connectMQTT){                            
        Serial.println("Could not connect MQTT.");
        Serial.println("Publish message NOK");
        return false;
      }
  }
  //If MQTT client is connected, publish message and prints results
  if (mqttClient.connected()){
    if (mqttClient.publish((char*)pubTopic.c_str(), (char*) message.c_str())) {
      Serial.println("Publish message OK");        
      return true;      
    } else {
      Serial.println("Publish state NOK");
      Serial.println("No MQTT connection.");        
    }
  }
  return false;   
}

/*
 * Method to publish MQTT alerts
 */
boolean pubAlert(){

  unsigned long dl = 5000;

  unsigned long currentMillis = millis();

  //Filter alerts, only send alerts every 5 seconds
  if (currentMillis - prevMs_alerted < dl) {
    return false;
  }

  //save the last time executed
  prevMs_alerted = currentMillis;

  //If MQTT client is not connected, connect 
  if (!connectMQTT()){
      delay(100);
      if (!connectMQTT){                            
        Serial.println("Could not connect MQTT.");
        Serial.println("Publish message NOK");
        return false;
      }
  }
  //If MQTT client is connected, publish message and prints results
  if (mqttClient.connected()){
    if (mqttClient.publish((char*)pubTopic.c_str(), (char*)alert.c_str())) {
      Serial.println("Publish message OK");        
      return true;      
    } else {
      Serial.println("Publish state NOK");
      Serial.println("No MQTT connection.");        
    }
  }
  return false;   
}
